﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
class ChartModel {

	private static var _JS_ARRAY_DELIMETER:String = "|";
	private static var _JS_ARRAY_ESCAPE:String = "\\";
	
	private static var _RGB_WHITE:Number = 0xFFFFFF;
	
	private static var _PARAM_DELIM:String = "&";
	private static var _PARAM_NAME_VALUE_DELIM:String = "=";
	
	private static var _MAX_VALUE_MODIFIER:Number = 1.5; // 50% increase
	
	// Required
	public var parameterCount:Number;
	// Required
 	public var parameterNames:Array;
	// Required
 	public var parameterValues:Array;
	// Required
 	public var presentation:String;

	// Calculated - using parameter values
	public var maxValue:Number;
	// Calculated - using parameter values
	public var minValue:Number;
	// Calculated - using parameter values
 	public var total:Number;

	// Optional - default: random colour palette
 	public var parameterColours:Array;
	// Optional - default: none
	public var onClickHandler:String;
	// Optional - default: auto-generated value
	public var chartID:String;
	// Optional - default: false
	public var isDebug:Boolean;
	// Optional - default: false
	public var animation:Boolean;
		
	public var enableClickHandler:Boolean;

		
	public function ChartModel() {
		// NO-OP
	}			
	
	public function loadParametersFromString(chartParameters:String):Void {
		// e.g. Chart parameters
		// _fparameterNames=High|Medium|Low|Unassigned&_fparameterValues=4|3|6|48
		// &_fparameterColours=R255G191B191|R250G234B187|R196G220B245|R240G240B240
		// &_fpresentation=bar&_fonClickHandler=true&_fisDebug=true
		// &_fanimation=true&_fchartID=com_ibm_team_dashboard_viewlets_web_ui_internal_StatisticsViewlet_0
		
		// Parse parameter pairs
		var parameterPairs:Array = chartParameters.split(ChartModel._PARAM_DELIM);
		
		for (var index:Number = 0; index < parameterPairs.length; index++) {
			// Parse pair (into name and value)
			var pair:Array = this._parseNameValue(parameterPairs[index]);
			// Validate pair
			if (pair.length != 2) {
				continue;
			}
			// Get parameter name
			var parameterName:String = String(pair[0]);
			// Get unescaped parameter value
			var parameterValue:String = unescape(String(pair[1]));
			// Set parameter on root
			_root[parameterName] = parameterValue;
		}
		
		this.loadParametersFromRoot();
	}
	
	private function _parseNameValue(parameterPair:String):Array {
		return parameterPair.split(ChartModel._PARAM_NAME_VALUE_DELIM, 2);
	}	
	
	public function loadParametersFromRoot():Void {
		// Parameter names
		this._loadParameterNames();
		// Parameter values
		this._loadParameterValues();
		// Max value
		// Min value
		// Parameter count
		// Total
		this._calculateValueStatistics();
		// Parameter colours
		this._loadParameterColours();
		// Presentation
		this._loadPresentation();
		// On click handler
		this._loadOnClickHandler();
		// Chart ID
		this._loadChartID();
		// Debug mode flag
		this._loadDebugFlag();
		// Show opening animation flag
		this._loadAnimationFlag();
	}
	
	public function isValid():Boolean {

		if (!this.presentation) {
			// Presentation
			return false;
		} else if (this.parameterCount <= 0) {
			// Parameter count
			return false;
		} else if (this.total < 0) {
			// Total
			return false;
		} else if (this.maxValue < 0) {
			// Max value
			return false;
		} else if (this.minValue < 0) {
			// Min value
			return false;
		} else if (this.parameterNames.length != this.parameterCount) {
			// Parameter names
			return false;
		} else if (this.parameterValues.length != this.parameterCount) {
			// Parameter values
			return false;
		} else if (this.parameterColours.length != this.parameterCount) {
			// Parameter colours
			return false;
		}

		for (var index:Number = 0; index < this.parameterCount; index++) {
			// Parameter names
			if (!this.parameterNames[index]) {
				return false;
			}
			// Parameter values
			if (this.parameterValues[index] < 0) {
				return false;
			}
			// Parameter colours
			if (this.parameterColours[index] < 0) {
				return false;
			}
		}
		
		return true;
	}
	
	private function _loadParameterNames():Void {
		if (_root._fparameterNames) {
			this.parameterNames = this._parseDelimitedString(_root._fparameterNames);			
			delete _root._fparameterNames;
		}
	}
	
	private function _loadParameterValues():Void {
		// TODO: Check if defined before processing
		var values:Array = _root._fparameterValues.split(ChartModel._JS_ARRAY_DELIMETER);
		delete _root._fparameterValues;		
		this.parameterValues = new Array();
		
		for (var index:Number = 0; index < values.length; index++) {
			this.parameterValues[index] = parseInt(values[index]);
		}
		
	}
	
	private function _loadParameterColours():Void {
		if (_root._fparameterColours) {
			this.parameterColours = this._parseColours();
			delete _root._fparameterColours;
		} else {
			this.parameterColours = ColourUtility.generateRandomColourPalette(this.parameterCount);
		}		
	}
	
	private function _loadPresentation():Void {
		// TODO: Check if defined before processing
		this.presentation = _root._fpresentation;
		delete _root._fpresentation;		
	}
	
	private function _loadOnClickHandler():Void {

		if (_root._fonClickHandler == "true") {
			this.enableClickHandler = true;
			delete _root._fonClickHandler;
		} else {
			this.enableClickHandler = false;
			delete _root._fonClickHandler;
			return;
		}
		
		var loadVars:LoadVars = new LoadVars();
		var context:ChartModel = this;
		
		loadVars.onLoad = function(success:Boolean) {
			if (!success) {
				return;
			}
			var valid:Boolean = context._isValidClickHandler(this.onClickHandler);
			if (!valid) {
				return;
			}
			context.onClickHandler = this.onClickHandler;
		}

		// Load the JavaScript click handler function name from a text file.
		loadVars.load("loadvars.txt");		
	}

	private function _isValidClickHandler(onClickHandler:String):Boolean {

		if (!onClickHandler) {
			return false;
		} else if (onClickHandler.indexOf("(") != -1) {
			return false;
		}
		return true;
	}	
	
	private function _loadChartID():Void {
		if (_root._fchartID) {
			this.chartID = _root._fchartID;
			delete _root._fchartID;
		} else {
			var date:Date = new Date();
			this.chartID = "chart_" + String(date.getTime());
		}
	}
	
	private function _loadDebugFlag():Void {
		// TODO: Check if defined before processing
		if (_root._fisDebug == "true") {
			this.isDebug = true;
		} else {
			this.isDebug = false;
		}		
		delete _root._fisDebug;
	}
	
	private function _loadAnimationFlag():Void {
		// TODO: Check if defined before processing
		if (_root._fanimation == "true") {
			this.animation = true;
		} else {
			this.animation = false;
		}		
		delete _root._fanimation;
	}
	
	private function _calculateValueStatistics():Void {
		
		// Parameter count
		this.parameterCount = this.parameterNames.length;
		
		var lTotal:Number = 0;
		var lMinValue:Number = Number.MAX_VALUE;
		var lMaxValue:Number = Number.MIN_VALUE;
		
		var lSameValues:Boolean = false;
		var lPreviousValue:Number;
		
		if (this.parameterValues.length > 1) {
			lSameValues = true;
			lPreviousValue = this.parameterValues[0];
		}
		
		for (var index:Number = 0; index < this.parameterCount; index++) {
			var lCurrentValue:Number = this.parameterValues[index];
			// Total
			lTotal = lTotal + lCurrentValue;
			// Min value
			if (lCurrentValue < lMinValue) {
				lMinValue = lCurrentValue;
			}
			// Max value
			if (lCurrentValue > lMaxValue) {
				lMaxValue = lCurrentValue;
			}
			// Same values flag
			if (lSameValues) {
				if (lCurrentValue != lPreviousValue) {
					lSameValues = false;
				}
				lPreviousValue = lCurrentValue;
			}
		}
		
		// When all the parameter values are identical, increase the maximum value
		// in order to prevent the chart shapes from occupying the entire
		// chart area (bar and column presentations)
		if (lSameValues) {
			lMaxValue = Math.round(lMaxValue * ChartModel._MAX_VALUE_MODIFIER);
		}
		
		// Max value
		this.maxValue = lMaxValue;
		// Min value
		this.minValue = lMinValue;
		// Total
		this.total = lTotal;
		
	}

	private function _parseDelimitedString(delimitedString:String):Array {
		
		// Only two characters can be escaped:
		// (1) <escape character>
		// (2) <delimiter character>
		
		var resultArray:Array = new Array();
		var resultEntry:String = "";
		var escapeMode:Boolean = false;
		
		// Process entire string - character by character
		for (var index:Number = 0; index < delimitedString.length; index++) {

			var character:String = delimitedString.charAt(index);
			
			if (escapeMode &&
				(character == ChartModel._JS_ARRAY_DELIMETER)) {
				// Swallow escape character
				// Add delimeter (current character) to entry
				resultEntry = resultEntry + character;
				// Turn off escape mode
				escapeMode = false;  
			} else if (escapeMode &&
				(character == ChartModel._JS_ARRAY_ESCAPE)) {
				// Swallow escape character
				// Add literal escape (current character) to entry
				resultEntry = resultEntry + character;
				// Turn off escape mode
				escapeMode = false;  
			} else if (escapeMode) {
				// Ignorable parse error: Dangling escape character
				// Swallow escape character
				// Add current character to entry
				resultEntry = resultEntry + character;
				// Turn off escape mode
				escapeMode = false;  
			} else if (character == ChartModel._JS_ARRAY_DELIMETER) {
				// Swallow delimeter (current character)
				// Add entry to array
				resultArray.push(resultEntry);
				// Reset entry
				resultEntry = "";
			} else if (character == ChartModel._JS_ARRAY_ESCAPE) {
				// Turn on escape mode
				escapeMode = true;  
			} else {
				// Add current character to entry
				resultEntry = resultEntry + character;
			}
		}
		
		// Add final entry to array
		resultArray.push(resultEntry);
		
		return resultArray;
	}
	
	private function _parseColours():Array {
		var colourStrings:Array = _root._fparameterColours.split(ChartModel._JS_ARRAY_DELIMETER);
		var colourHexValues:Array = new Array();
		
		for (var index:Number = 0; index < colourStrings.length; index++) {
			colourHexValues[index] = this._parseRGBValue(colourStrings[index]);
		}
		
		return colourHexValues;
	}
	
	// TODO: Move to ColourUtility?
	private function _parseRGBValue(value:String):Number {
		
		var RGBColour:Object = new Object();
		
		// e.g. R128G100B23		
		
		if (value.charAt(0) != "R") {
			return ChartModel._RGB_WHITE;
		}

		// Get red value
		
		var startIndex:Number = 1;
		var endIndex:Number = -1;
		
		for (var index:Number = startIndex; index < value.length; index++) {
			if (value.charAt(index) == "G") {
				endIndex = index;
				break;
			}
		}
		
		if (endIndex == -1) {
			return ChartModel._RGB_WHITE;
		}
		RGBColour.red = parseInt(value.substring(startIndex, endIndex));
		
		// Get green value

		startIndex = endIndex + 1;
		endIndex = -1;

		for (var index:Number = startIndex; index < value.length; index++) {
			if (value.charAt(index) == "B") {
				endIndex = index;
				break;
			}
		}
		
		if (endIndex == -1) {
			return ChartModel._RGB_WHITE;
		}
		RGBColour.green = parseInt(value.substring(startIndex, endIndex));
		
		// Get blue value

		startIndex = endIndex + 1;
		
		RGBColour.blue = parseInt(value.substring(startIndex, value.length));
		
		// Convert
		
		var hex = ColourUtility.convertRGBToHex(RGBColour);
		
		return hex;
		
	}
	
	public function getMaxParameterNameWidth(textFormat:TextFormat):Number {

		var maxWidth:Number = 0;
		
		for (var index:Number = 0; index < this.parameterCount; index++) {
			var name:String = this.parameterNames[index];
			var properties:Object = textFormat.getTextExtent(name);
			
			if (properties.textFieldWidth > maxWidth) {
				maxWidth = properties.textFieldWidth;
			}
		}
	
		return maxWidth;
	}	
	
	public function destroy():Void {
		// TODO: Make interface method?

		delete this.presentation;
		
		for (var index:Number = 0; index < this.parameterCount; index++) {
			// Parameter names
			delete this.parameterNames[index];
			// Parameter values
			delete this.parameterValues[index];
			// Parameter colours
			delete this.parameterColours[index];
		}
		
		delete this.parameterCount;
		delete this.parameterNames;
		delete this.parameterValues;
		delete this.parameterColours;
		
		delete this.maxValue;
		delete this.minValue;
		delete this.total;
		
		delete this.onClickHandler;
		delete this.chartID;
		delete this.isDebug;
		delete this.animation;		
	}
	
}