/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2008, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.page.FavoritesController");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.Preference");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "FavoritesMessages");

(function(){

var messages;
var cq = com.ibm.rational.cm.web;
var preference = com.ibm.rational.cm.preference;

// static strings
var PREF_HIDDEN = "HIDDEN_FIELDS";
var PREF_FAVORITES = "FAVORITES";
var TOPIC_FAV_INIT = "CQ_FAVORITE_INIT";
var TOPIC_FAV_ADDED = "CQ_FAVORITE_ADDED";
var TOPIC_FAV_REMOVED = "CQ_FAVORITE_REMOVED";
var favObj, _init, _loading;

dojo.declare(
	"com.ibm.rational.cm.web.ui.page.FavoritesController",
	null,
{
	constructor: function(){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "FavoritesMessages");
		_init = false;
	},
	
	/*
	 * Loads favorites from preferences
	 * argument:
	 * {
	 *	onBegin: function		called when starting to fetch preference
	 *	onLoad: function(Array)	called when preference is loaded 
	 *	onCancel: function		called when cancel is clicked in login dialog
	 *	forceLoad: boolean		true if forcing loading preference even if it is already loaded
	 * }
	 */
	loadFavorites: function(/* Object */ obj){
		var self = this;
		preference.process({
			onBegin: obj.onBegin,
			onLoad: function(){
				var items = self.getFavoriteItems();
				var onLoad = obj.onLoad;
				if(onLoad && dojo.isFunction(onLoad)){
					onLoad(items);
				}
				if(!_init){
					_init = true;
					dojo.publish(TOPIC_FAV_INIT, items);
				}
			},
			onCancel: obj.onCancel,
			forceLoad: obj.forceLoad
		});
	},
	
	/*
	 * Returns an array of objects in the following form:
	 *{
	 *	id: String (stableLocation),
	 *	type: String (LIST|RECORD|CHART|REPORT)
	 *	label: String (label)
	 *}
	 */
	getFavoriteItems: function(){
		var favPref = preference.getValue(PREF_HIDDEN, PREF_FAVORITES);
		if(favPref){
			favObj = dojo.fromJson(decodeURIComponent(favPref));
			var idProp = favObj.identifier;
			var labelProp = favObj.label;
			return dojo.map(favObj.items, function(item){
				return {
					id: item[idProp],
					type: item.type,
					label: item[labelProp],
					masterReplica: item.masterReplica
				};
			});
		}else{
			favObj = {
				identifier: "stableLocation",
				label: "title",
				items: []
			}
			return [];
		}
	},

	/*
	 * Checks if specified resource already exists in favorites
	 * and requests to add it to favorites if it doesn't exist
	 */
	addFavorite: function(message){
		var title = message.title;
		if(this.isFavoriteExist(message.stableLocation)){
			alert(dojo.string.substitute(messages.FavAlreadyExist, [title]));
			return;
		}
		if(!title || title.length === 0){
			this._retrieveItemProperties(message, this._doAddFavorite);
		}else{
			this._doAddFavorite(message);
		}	
	},
	
	/*
	 * Checks if specify resource already exists in favorites
	 */
	isFavoriteExist: function(/* String */ id){
		return dojo.some(favObj.items, function(item){
			return (id == item[favObj.identifier]);
		});
	},
	
	/*
	 * Get displayName and recordType for given stableLocation of a query
	 */
	_retrieveItemProperties: function(message, callback){
		var self = this;
		dojo["require"]("com.ibm.rational.cm.web.ui.page.PropertyManager");
		com.ibm.rational.cm.web.ui.page.propManager.getProperties({
			resourceId: message.stableLocation,
			onLoad: function(props){
				message.title = (props.type == "RECORD") ? 
					props.recordType + "/" + props.displayName : props.displayName;
				message.type = props.type;
				callback.call(self, message);
			}
		});
	},
	
	/*
	 * Actually save the added favorite information to preferences
	 */
	_doAddFavorite: function(message){
		if(!_init || !favObj || !favObj.items || !dojo.isArray(favObj.items)){
			console.error("Favorite controller is not initialized");	 //$NON-NLS-1$
			return;
		}
		
		// save it to preference
		var self = this;
		var favObjCopy = dojo.clone(favObj);
		favObjCopy.items.push(message);
		preference.setValue(
			PREF_HIDDEN,
			PREF_FAVORITES,
			encodeURIComponent(dojo.toJson(favObjCopy)),
			{
				onSaved: function(){
					// update favObj after successfully updating preference on server
					favObj.items.push(message);

					dojo.publish(TOPIC_FAV_ADDED, [{
						id: message.stableLocation,
						type: message.type,
						label: message.title,
						masterReplica: message.masterReplica
					}]);
				},
				statusMessage: messages.AddingFav
			}
		);
	},
	
	/*
	 * Confirm deleting favorites, and request to delete if user pressed OK
	 */
	removeFavorite: function(message){
		if(!confirm(dojo.string.substitute(messages.ConfirmRemove, [message.title]))){ return; }
	
		this._doRemoveFavorite(message.id);
	},
	
	/*
	 * Actually remove the specified resource from favorites
	 */
	_doRemoveFavorite: function(id){
		if(!_init || !favObj || !favObj.items || !dojo.isArray(favObj.items)){
			console.error("Favorite controller is not initialized");	//$NON-NLS-1$
			return;
		}
		
		// save it to preference
		var self = this;
		// Defect 79866
		// make a copy of favObj, so if fail to update preference, we can keep favObj consistent 
		var favObjCopy = dojo.clone(favObj);
		for(var i = favObjCopy.items.length-1; i >= 0; i--){
			if(favObjCopy.items[i].stableLocation == id){
				favObjCopy.items.splice(i, 1);
				preference.setValue(
					PREF_HIDDEN,
					PREF_FAVORITES,
					encodeURIComponent(dojo.toJson(favObjCopy)),
					{
						onSaved: function(){
							dojo.publish(TOPIC_FAV_REMOVED, [{
								id: id
							}]);
							// update favObj after successfully update preference on server
							favObj.items.splice(i, 1);
						},
						statusMessage: messages.RemovingFav
					}
				);
				break;
			}
		}
	}
});

})();

com.ibm.rational.cm.favoritesController = new com.ibm.rational.cm.web.ui.page.FavoritesController();